import httpx
from httpx import ConnectError
from httpx import ConnectTimeout
from fake_useragent import UserAgent
from scan.response import Response
from scan.common import logger


class Downloader(object):
    def __init__(self):
        self.client = None
        self.ua = UserAgent()

    async def gen_headers(self):
        try:
            ua = self.ua.random
            headers = {
                'accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;'
                          'q=0.8,application/signed-exchange;v=b3;q=0.9',
                'accept-encoding': 'gzip, deflate, br',
                'accept-language': 'en;q=0.9',
                'user-agent': ua
            }
            return headers
        except Exception as e:
            await logger.error(f'gen ua error: {e}')

    async def close(self):
        try:
            await self.client.aclose()
        except:
            pass

    @staticmethod
    async def log_request(request):
        pass

    @staticmethod
    async def log_response(response):
        """
        日志钩子
        """
        request = response.request
        if response.status_code not in [200, 301, 302]:
            await logger.error(f'{response.status_code}  {request.url}')
        else:
            await logger.info(f'{response.status_code}  {request.url}')

    async def request(self, url, params=None, headers=None, cookies=None, auth=None, proxies=None, allow_redirects=True,
                      content=None, data=None, files=None, json=None, timeout=30, cycle=3):
        if data or json or content:
            method = 'POST'
        else:
            method = 'GET'
        if not headers:
            headers = await self.gen_headers()
        for _ in range(cycle):
            try:
                if not proxies:
                    if not self.client:
                        self.client = httpx.AsyncClient(event_hooks={'response': [self.log_response]})
                    resp = await self.client.request(
                        method=method, url=url, content=content, data=data, files=files, json=json, params=params,
                        headers=headers, cookies=cookies, auth=auth, allow_redirects=allow_redirects, timeout=timeout
                    )
                    response = Response(resp)
                    return response
                else:
                    async with httpx.AsyncClient(proxies=proxies, event_hooks={'response': [self.log_response]}) as client:
                        resp = await client.request(
                            method=method, url=url, content=content, data=data, files=files, json=json, params=params,
                            headers=headers, cookies=cookies, auth=auth, allow_redirects=allow_redirects,
                            timeout=timeout
                        )
                        response = Response(resp)
                        return response
            except ConnectError as e:
                await logger.error(f'Failed to request {url}  ConnectError:{e}')
            except ConnectTimeout as e:
                await logger.error(f'Failed to request {url}  ConnectTimeout:{e}')
            except Exception as e:
                await logger.error(f'Failed to request {url}  {e}')







import json
import aio_pika
import asyncio
from aio_pika.pool import Pool
from scan.common import logger
from json import JSONDecodeError


class RabbitMQ:
    def __init__(self, **kwargs):
        self.host = kwargs.get('host') or 'localhost'
        self.port = kwargs.get('port') or 5672
        self.user = kwargs.get('user') or 'root'
        self.password = kwargs.get('password') or 'root'
        self.virtualhost = kwargs.get('virtualhost') or '/'
        self.channel_pool = None
        self.loop = None
        self.max_pool_size = kwargs.get('max_pool_size') or 20

    async def init(self, max_pool_size=None):
        if max_pool_size and isinstance(max_pool_size, int):
            self.max_pool_size = max_pool_size
        await self.get_channel_pool()

    async def get_channel_pool(self):
        try:
            if self.loop or self.channel_pool:
                try:
                    await self.channel_pool.close()
                    self.loop.close()
                except Exception as e:
                    await logger.error(f'close error: {e}')
            self.loop = asyncio.get_event_loop()

            async def get_connection():
                return await aio_pika.connect_robust(host=self.host, port=int(self.port), login=self.user,
                                                     password=self.password, virtualhost=self.virtualhost)
            connection_pool = Pool(get_connection, max_size=int(self.max_pool_size), loop=self.loop)

            async def get_channel() -> aio_pika.Channel:
                async with connection_pool.acquire() as connection:
                    return await connection.channel()

            self.channel_pool = Pool(get_channel, max_size=self.max_pool_size, loop=self.loop)
        except Exception as e:
            await logger.error(f'get channel pool error: {e}')
            await asyncio.sleep(30)

    async def consume(self, queue_name, arguments=None):
        """
        :param arguments: mq绑定参数
        :param queue_name: 队列名
        :return:
        """
        try:
            async with self.channel_pool.acquire() as channel:
                for _ in range(3):
                    try:
                        await channel.set_qos(1)
                        queue = await channel.declare_queue(queue_name, durable=True, arguments=arguments,
                                                            auto_delete=False)
                        async with queue.iterator() as queue_iter:
                            async for message in queue_iter:
                                body = None
                                try:
                                    body = message.body
                                    data_dict = json.loads(body.decode())
                                    return data_dict, message
                                except JSONDecodeError as e:
                                    await logger.error(f'JSONDecoder error: {e}  data:{body}')
                                except Exception as e:
                                    await logger.error(f'consume msg error: {e}')
                                    await self.get_channel_pool()
                    except Exception as e:
                        await logger.error(f'consume error: {e}')
                        await self.get_channel_pool()
        except Exception as e:
            await logger.error(f'consume process error: {e}')
            await asyncio.sleep(30)
            await self.get_channel_pool()

    async def publish(self, data, routing_key, priority=None):
        """
        :param priority: 消息优先级
        :param data: 要发送的数据
        :param routing_key: 队列名
        :return:
        """
        try:
            async with self.channel_pool.acquire() as channel:
                data = json.dumps(data)
                for _ in range(3):
                    try:
                        await channel.default_exchange.publish(aio_pika.Message(body=data.encode(), priority=priority),
                                                               routing_key=routing_key)
                        return
                    except Exception as e:
                        await logger.error(f'publish error: {e}')
                        await self.get_channel_pool()
        except Exception as e:
            await logger.error(f'publish process error: {e}')
            await asyncio.sleep(30)
            await self.get_channel_pool()

    async def purge(self, queue_name, arguments=None):
        """
        :param arguments: 绑定队列参数
        :param queue_name: 要清空的队列名
        :return:
        """
        try:
            async with self.channel_pool.acquire() as channel:
                for _ in range(3):
                    try:
                        await channel.set_qos(1)
                        queue = await channel.declare_queue(queue_name, durable=True, arguments=arguments,
                                                            auto_delete=False)
                        await queue.purge()
                        return
                    except Exception as e:
                        await logger.error(f'purge error: {e}')
                        await self.get_channel_pool()
        except Exception as e:
            await logger.error(f'purge process error: {e}')
            await asyncio.sleep(30)
            await self.get_channel_pool()

    async def message_count(self, queue_name, arguments=None):
        try:
            async with self.channel_pool.acquire() as channel:
                for _ in range(3):
                    try:
                        await channel.set_qos(1)
                        queue = await channel.declare_queue(queue_name, durable=True, arguments=arguments,
                                                            auto_delete=False)
                        count = queue.declaration_result.message_count
                        return count
                    except Exception as e:
                        await logger.error(f'message_count error: {e}')
                        await self.get_channel_pool()
        except Exception as e:
            await logger.error(f'message_count process error: {e}')
            await asyncio.sleep(30)
            await self.get_channel_pool()


__all__ = RabbitMQ











import json
from bs4 import BeautifulSoup
from scan.common import logger


class Response:
    def __init__(self, req_resp):
        self.req_resp = req_resp
        self.status_code = req_resp.status_code
        self.content = req_resp.content

    def json(self):
        try:
            return json.loads(self.content)
        except Exception as e:
            logger.error(f'格式化json异常:{e}, 数据:{self.content}')

    def soup(self):
        try:
            soup = BeautifulSoup(self.content, 'lxml')
            return soup
        except Exception as e:
            logger.error(f'格式化soup异常:{e}, 数据:{self.content}')

    def text(self):
        try:
            return self.content.decode()
        except Exception as e:
            logger.error(f'格式化text异常:{e}, 数据:{self.content}')






import os
import sys
import asyncio
import datetime

from aiologger import Logger
from aiologger.utils import bind_loop
from aiologger.levels import LogLevel
from aiologger.formatters.base import Formatter
from aiologger.handlers.files import AsyncFileHandler
from aiologger.handlers.streams import AsyncStreamHandler


class AioLogger(object):
    def __init__(self, name=None, loop=None):
        # day_date = datetime.datetime.now().strftime('%Y-%m-%d')
        # log_path = os.path.join(os.path.dirname(os.path.abspath(sys.argv[0])), 'logs/')
        # if not os.path.exists(log_path):
        #     os.makedirs(log_path)

        if loop is None:
            loop = asyncio.new_event_loop()
            asyncio.set_event_loop(loop)
        self.loop = loop
        # formatter = Formatter('[%(asctime)s] %(filename)s -> line:%(lineno)d [%(levelname)s] %(message)s')
        formatter = Formatter('%(asctime)s %(levelname)s %(message)s')

        logger = Logger(name=name, level=LogLevel.DEBUG)
        ash = bind_loop(AsyncStreamHandler, {})(stream=sys.stderr, level=LogLevel.DEBUG, formatter=formatter)
        logger.add_handler(ash)

        # log_name = f'{log_path}{name + ".log"}'
        # afh = AsyncFileHandler(log_name, 'a', encoding='utf-8')
        # afh.formatter = formatter

        self.logger = logger
        # self.logger.add_handler(afh)


def logger(name):
    return AioLogger(name).logger



from scan.downloade.downloader import Downloader


class Crawl:
    def __init__(self):
        self.downloader = Downloader()

    async def fetch(self,  url, params=None, data=None, files=None, json=None, content=None,headers=None, cookies=None,
                    auth=None, proxies=None, allow_redirects=True, timeout=30, cycle=3):
        res = await self.downloader.request(
            url, params=params, data=data, files=files, json=json,content=content, headers=headers, proxies=proxies,
            cookies=cookies, auth=auth, allow_redirects=allow_redirects, timeout=timeout, cycle=cycle
        )
        return res

    async def close(self):
        await self.downloader.close()


crawl = Crawl()
__all__ = crawl
